{: This unit contains the TJPMColorComboBox which is a color selection combo-box
 which is independant of any third-party components.<p>
}
{Release notes
1-31-2000 started this component by swiping all of the component code in jfntecbo
          I had to make adjustments to remove ESBPCS-specific features
          published ItemHeight and Font properties
          Moved component to "JPM Open"
2-2-2000 fixed a bug in dynamc control creation where items were not listed
7-28-2000 Cleaned-up the code
          colors are now in resource strings to facilitate internationalization}
unit JPMColorComboBox;

interface

{$I JPM.INC}

uses Classes, Controls,
     Graphics, JPMColorListObject,
     Stdctrls, Windows;

type
     TJPMColorComboBox = class (TCustomComboBox)
     private
          { Private fields of TJPMColorComboBox }
          FColorSelObject: TJPMColorListObject;
          { Private methods of TJPMColorComboBox }
          {: Method to set variable and property values and create objects }
          procedure AutoInitialize;
          {: Method to free any objects created by AutoInitialize }
          procedure AutoDestroy;
          {: Read method for property SelectColor }
          function GetSelectColor: TColor;
          {: Write method for property SelectColor }
          procedure SetSelectColor (Value: TColor);
          {: Read method for property ShowSystemColors }
          function GetShowSystemColors: Boolean;
          {: Write method for property AdditionalColors}
          procedure SetShowAdditionalColors (Value: Boolean);
          {: Read method for property AdditionalColors}
          function GetShowAdditionalColors: Boolean;
          {: Write method for property ShowSystemColors }
          procedure SetShowSystemColors (Value: Boolean);
          {: Write method for property ShowCustomColor }
          procedure SetShowCustomColor (Value: Boolean);
          {: Read method for property ShowCustomColor }
          function GetShowCustomColor: Boolean;
          {: PreviewBoxWidth write method for PreviewBoxWidth }
          procedure SetPreviewBoxWidth (Value: Integer);
          {: PreviewBoxWidth read method for PreviewBoxWidth }
          function GetPreviewBoxWidth: Integer;
          function GetShowBox: Boolean;
          function GetShowBoxBorder: Boolean;
          function GetShowStyledText: Boolean;
          function GetTextOnColorBackground: Boolean;
          procedure SetShowBox (const Value: Boolean);
          procedure SetShowBoxBorder (const Value: Boolean);
          procedure SetShowStyledText (const Value: Boolean);
          procedure SetTextOnColorBackground (const Value: Boolean);
     protected
          { Protected fields of TJPMColorComboBox }
          procedure SetParent (AParent: TWinControl); override;
          procedure Change; override;
          procedure Click; override;
          procedure KeyPress (var Key: Char); override;
          procedure Loaded; override;
          {: This places the items in the ComboBox}
          procedure PopulateComboBox; virtual;
          function GetJPMVersion: string;
          procedure SetJPMVersion (const Value: string);
     public
          { Public fields and properties of TJPMColorComboBox }
          { Public methods of TJPMColorComboBox }
          {: Creates the comobobox}
          constructor Create (AOwner: TComponent); override;
          {: Destroys the combobox}
          destructor Destroy; override;
          {: override to display colors in box }
          procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
          {: method for calling ColorSelection Dialog-box }
          function ExecuteDialog: Boolean;
     published
          { Published properties of TJPMColorComboBox }
          property ItemHeight default 15;
          property Font;
          property TabOrder;
          property TabStop;
          property Visible;
          property OnChange;
          property OnClick;
          property OnDblClick;
          property OnDragDrop;
          property OnEnter;
          property OnExit;
          property OnKeyDown;
          property OnKeyPress;
          property OnKeyUp;
          property OnMouseDown;
          property OnMouseMove;
          property OnMouseUp;
          {: Color which was selected in the combo-box }
          property SelectColor: TColor read GetSelectColor write SetSelectColor;
          {: Show system colors such as clWIndows }
          property ShowSystemColors: Boolean read GetShowSystemColors
               write SetShowSystemColors default True;
          {: Show additional colors such as Pink, Lemon, etc.}
          property ShowAdditionalColors: Boolean read GetShowAdditionalColors write
               SetShowAdditionalColors default True;
          {: Width of the ComboBox}
          property Width default 126;
          {: Show the Custom Color item which pops-up the color selection dialog}
          property ShowCustomColor: Boolean read GetShowCustomColor
               write SetShowCustomColor default true;
          {: This is width of the color-preview box which displays the color}
          property PreviewBoxWidth: Integer read GetPreviewBoxWidth
               write SetPreviewBoxWidth default 40;
          {: Show a border around the preview box}
          property ShowBoxBorder: Boolean read GetShowBoxBorder write SetShowBoxBorder;
          {: Show a preview box in ListControl}
          property ShowBox: Boolean read GetShowBox write SetShowBox;
          {: Show the text on using the color as a background}
          property TextOnColorBackground: Boolean read GetTextOnColorBackground write SetTextOnColorBackground;
          {: ShowStyledText variable indicates that we should draw text outside the preview with a particular
           Font or color}
          property ShowStyledText: Boolean read GetShowStyledText write SetShowStyledText;

          {: Displays the Current Version of the Component. }
          property Version: string read GetJPMVersion write SetJPMVersion stored false;
     end;

implementation
uses SysUtils, JPMConst;

{ Method to set variable and property values and create objects }

procedure TJPMColorComboBox.AutoInitialize;
begin
     Sorted := False;
     Style := csOwnerDrawFixed;
     FColorSelObject := TJPMColorListObject.Create (Items);
     FColorSelObject.ShowAdditionalColors := True;
     FColorSelObject.ShowSystemColors := True;
     FColorSelObject.ShowCustomColor := True;
     FColorSelObject.ShowBoxWidth := 40;
     ItemHeight := 15;
     Width := 126;
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }

procedure TJPMColorComboBox.AutoDestroy;
begin
     FColorSelObject.Free;
end; { of AutoDestroy }

function TJPMColorComboBox.GetJPMVersion: string;
begin
     Result := JPMVersion;
end;

procedure TJPMColorComboBox.SetJPMVersion (const Value: string);
begin
     // Do nothing
end;

{ Read method for property SelectColor }

function TJPMColorComboBox.GetSelectColor: TColor;
begin
     Result := FColorSelObject.SelectColor;
end;

{ Write method for property SelectColor }

procedure TJPMColorComboBox.SetSelectColor (Value: TColor);
begin
     FColorSelObject.SelectColor := Value;
     FColorSelObject.Items.Objects [FColorSelObject.Items.Count - 1] := TObject (Value);

     ItemIndex := Items.IndexOfObject (TObject (Value));
     if ItemIndex < 0 then
          ItemIndex := Items.Count - 1;
end;

{ Read method for property ShowSystemColors }

function TJPMColorComboBox.GetShowSystemColors: Boolean;
begin
     Result := FColorSelObject.ShowSystemColors;
end;

{ Write method for property ShowSystemColors }

procedure TJPMColorComboBox.SetShowSystemColors (Value: Boolean);
begin
     FColorSelObject.ShowSystemColors := Value;
     {We do not want to create the Color items if we are loading lest
     we create problems.  In any case, it would only duplicate a single action}
     if not (csReading in ComponentState) then
          PopulateComboBox;
end;

{ Override OnChange handler from TESBCustomComboBox }

procedure TJPMColorComboBox.Change;
begin
     {if last - show color dialog-box if appropriate}
     if (ItemIndex = (Items.Count - 1)) then
          ExecuteDialog //show color dialog-box
     else if ItemIndex >= 0 then
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //reflect the color in SelectColor
     inherited Change;
end;

{ Override OnClick handler from TESBCustomComboBox }

procedure TJPMColorComboBox.Click;
begin
     { Code to execute before activating click
   behavior of component's parent class }

     { Activate click behavior of parent }
     inherited Click;

     { Code to execute after click behavior
       of parent }
end;

{ Override OnKeyPress handler from TESBCustomComboBox }

procedure TJPMColorComboBox.KeyPress (var Key: Char);
const
     TabKey = Char (VK_TAB);
     EnterKey = Char (VK_RETURN);
begin
     { Key contains the character produced by the keypress.
      It can be tested or assigned a new value before the
      call to the inherited KeyPress method.  Setting Key
      to #0 before call to the inherited KeyPress method
      terminates any further processing of the character. }

     { Activate KeyPress behavior of parent }
     inherited KeyPress (Key);

     { Code to execute after KeyPress behavior of parent }
end;

constructor TJPMColorComboBox.Create (AOwner: TComponent);
begin
     { Call the Create method of the parent class }
     inherited Create (AOwner);

     { AutoInitialize sets the initial values of variables and      }
     { properties; also, it creates objects for properties of       }
     { standard Delphi object types (e.g., TFont, TTimer,           }
     { TPicture) and for any variables marked as objects.           }
     { AutoInitialize method is generated by Component Create.      }
     AutoInitialize;
     { Code to perform other tasks when the component is created }
end;

destructor TJPMColorComboBox.Destroy;
begin
     { AutoDestroy, which is generated by Component Create, frees any   }
     { objects created by AutoInitialize.                               }
     AutoDestroy;

     { Here, free any other dynamic objects that the component methods  }
     { created but have not yet freed.  Also perform any other clean-up }
     { operations needed before the component is destroyed.             }

     { Last, free the component by calling the Destroy method of the    }
     { parent class.                                                    }
     inherited Destroy;
end;

{ //override to display colors in box }

procedure TJPMColorComboBox.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
     if (odFocused in State) or (odSelected in State) then
     begin
          Canvas.Font.Color := clHighLightText;
          Canvas.Brush.Color := clHighLight;
     end //if (odFocused in State) or (odSelected in State) then
     else
     begin
          Canvas.Font.Color := Font.Color;
          Canvas.Brush.Color := Color;
     end; //else ... if (odFocused in State) or (odSelected in State) then
     Canvas.FillRect (Rect);
     {Color preview rectange}
     FColorSelObject.DrawBox (Canvas, Index, Rect, State);
     inherited DrawItem (Index, Rect, State);
end;

{ //Set Up color selection for combo-box }

procedure TJPMColorComboBox.Loaded;
begin
     inherited Loaded;
     PopulateComboBox;
end;

//this loads the colors into the listbox

procedure TJPMColorComboBox.PopulateComboBox;
begin
     FColorSelObject.PopulateComboBox;
     ItemIndex := Items.IndexOfObject (TObject (FColorSelObject.SelectColor));
end;

procedure TJPMColorComboBox.SetShowCustomColor (Value: Boolean);
begin
     FColorSelObject.ShowCustomColor := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

{Write method for property ShowAdditionalColors }

procedure TJPMColorComboBox.SetShowAdditionalColors (Value: Boolean);
begin
     FColorSelObject.ShowAdditionalColors := Value;
     if not (csReading in ComponentState) then
          PopulateComboBox; //repopulate list
end;

function TJPMColorComboBox.ExecuteDialog: Boolean;
{The key assumption is that "Custom Color" is the last item in the list}
begin
     Result := False; {assume false for expediancy}
     if FColorSelObject.ExecDialogBoxEnh (JPMCustomColors) then
          ItemIndex := Items.Count - 1 {Select "Custom Color" for user if this was called independantly of the combobox}
     else
          FColorSelObject.SelectColor := TColor (Items.Objects [ItemIndex]); //make sure that the original color is preserved
end;

function TJPMColorComboBox.GetShowAdditionalColors: Boolean;
begin
     Result := FColorSelObject.ShowAdditionalColors;
end;

function TJPMColorComboBox.GetShowCustomColor: Boolean;
begin
     Result := FColorSelObject.ShowCustomColor;
end;

function TJPMColorComboBox.GetPreviewBoxWidth: Integer;
begin
     Result := FColorSelObject.ShowBoxWidth;
end;

procedure TJPMColorComboBox.SetPreviewBoxWidth (Value: Integer);
begin
     FColorSelObject.ShowBoxWidth := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is changed
end;

procedure TJPMColorComboBox.SetParent (AParent: TWinControl);
begin
     inherited;
     if AParent <> nil then
          PopulateComboBox;
end;

function TJPMColorComboBox.GetShowBox: Boolean;
begin
     Result := FColorSelObject.ShowBox;
end;

function TJPMColorComboBox.GetShowBoxBorder: Boolean;
begin
     Result := FColorSelObject.ShowBoxBorder;
end;

function TJPMColorComboBox.GetShowStyledText: Boolean;
begin
     Result := FColorSelObject.ShowStyledText;
end;

function TJPMColorComboBox.GetTextOnColorBackground: Boolean;
begin
     Result := FColorSelObject.TextOnColorBackground;
end;

procedure TJPMColorComboBox.SetShowBox (const Value: Boolean);
begin
     FColorSelObject.ShowBox := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMColorComboBox.SetShowBoxBorder (const Value: Boolean);
begin
     FColorSelObject.ShowBoxBorder := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click

end;

procedure TJPMColorComboBox.SetShowStyledText (const Value: Boolean);
begin
     FColorSelObject.ShowStyledText := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMColorComboBox.SetTextOnColorBackground (const Value: Boolean);
begin
     FColorSelObject.TextOnColorBackground := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

end.
